@@ -18,9 +18,12 @@ import hashlib
 import logging
 import math
 import os
+import shlex
 import socket
 import time
 
+from keystoneauth1.access import service_catalog as keystone_sc
+from keystoneauth1 import exceptions as keystone_exc
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_utils import units
@@ -31,17 +34,17 @@ import glance_store.driver
 from glance_store import exceptions
 from glance_store.i18n import _, _LE, _LW, _LI
 import glance_store.location
-from keystoneclient import exceptions as keystone_exc
-from keystoneclient import service_catalog as keystone_sc
 
 try:
     from cinderclient import exceptions as cinder_exception
     from cinderclient.v2 import client as cinderclient
     from os_brick.initiator import connector
+    from oslo_privsep import priv_context
 except ImportError:
     cinder_exception = None
     cinderclient = None
     connector = None
+    priv_context = None
 
 
 CONF = cfg.CONF
@@ -50,49 +53,258 @@ LOG = logging.getLogger(__name__)
 _CINDER_OPTS = [
     cfg.StrOpt('cinder_catalog_info',
                default='volumev2::publicURL',
-               help=_('Info to match when looking for cinder in the service '
-                      'catalog. Format is : separated values of the form: '
-                      '<service_type>:<service_name>:<endpoint_type>')),
+               help=_("""
+Information to match when looking for cinder in the service catalog.
+
+When the ``cinder_endpoint_template`` is not set and any of
+``cinder_store_auth_address``, ``cinder_store_user_name``,
+``cinder_store_project_name``, ``cinder_store_password`` is not set,
+cinder store uses this information to lookup cinder endpoint from the service
+catalog in the current context. ``cinder_os_region_name``, if set, is taken
+into consideration to fetch the appropriate endpoint.
+
+The service catalog can be listed by the ``openstack catalog list`` command.
+
+Possible values:
+    * A string of of the following form:
+      ``<service_type>:<service_name>:<interface>``
+      At least ``service_type`` and ``interface`` should be specified.
+      ``service_name`` can be omitted.
+
+Related options:
+    * cinder_os_region_name
+    * cinder_endpoint_template
+    * cinder_store_auth_address
+    * cinder_store_user_name
+    * cinder_store_project_name
+    * cinder_store_password
+
+""")),
     cfg.StrOpt('cinder_endpoint_template',
-               help=_('Override service catalog lookup with template for '
-                      'cinder endpoint.')),
+               default=None,
+               help=_("""
+Override service catalog lookup with template for cinder endpoint.
+
+When this option is set, this value is used to generate cinder endpoint,
+instead of looking up from the service catalog.
+This value is ignored if ``cinder_store_auth_address``,
+``cinder_store_user_name``, ``cinder_store_project_name``, and
+``cinder_store_password`` are specified.
+
+If this configuration option is set, ``cinder_catalog_info`` will be ignored.
+
+Possible values:
+    * URL template string for cinder endpoint, where ``%%(tenant)s`` is
+      replaced with the current tenant (project) name.
+      For example: ``http://cinder.openstack.example.org/v2/%%(tenant)s``
+
+Related options:
+    * cinder_store_auth_address
+    * cinder_store_user_name
+    * cinder_store_project_name
+    * cinder_store_password
+    * cinder_catalog_info
+
+""")),
     cfg.StrOpt('cinder_os_region_name', deprecated_name='os_region_name',
-               help=_('Region name of this node. If specified, it will be '
-                      'used to locate OpenStack services for stores.')),
+               default=None,
+               help=_("""
+Region name to lookup cinder service from the service catalog.
+
+This is used only when ``cinder_catalog_info`` is used for determining the
+endpoint. If set, the lookup for cinder endpoint by this node is filtered to
+the specified region. It is useful when multiple regions are listed in the
+catalog. If this is not set, the endpoint is looked up from every region.
+
+Possible values:
+    * A string that is a valid region name.
+
+Related options:
+    * cinder_catalog_info
+
+""")),
     cfg.StrOpt('cinder_ca_certificates_file',
-               help=_('Location of ca certificates file to use for cinder '
-                      'client requests.')),
+               help=_("""
+Location of a CA certificates file used for cinder client requests.
+
+The specified CA certificates file, if set, is used to verify cinder
+connections via HTTPS endpoint. If the endpoint is HTTP, this value is ignored.
+``cinder_api_insecure`` must be set to ``True`` to enable the verification.
+
+Possible values:
+    * Path to a ca certificates file
+
+Related options:
+    * cinder_api_insecure
+
+""")),
     cfg.IntOpt('cinder_http_retries',
+               min=0,
                default=3,
-               help=_('Number of cinderclient retries on failed http calls')),
+               help=_("""
+Number of cinderclient retries on failed http calls.
+
+When a call failed by any errors, cinderclient will retry the call up to the
+specified times after sleeping a few seconds.
+
+Possible values:
+    * A positive integer
+
+Related options:
+    * None
+
+""")),
     cfg.IntOpt('cinder_state_transition_timeout',
+               min=0,
                default=300,
-               help=_('Time period of time in seconds to wait for a cinder '
-                      'volume transition to complete.')),
+               help=_("""
+Time period, in seconds, to wait for a cinder volume transition to
+complete.
+
+When the cinder volume is created, deleted, or attached to the glance node to
+read/write the volume data, the volume's state is changed. For example, the
+newly created volume status changes from ``creating`` to ``available`` after
+the creation process is completed. This specifies the maximum time to wait for
+the status change. If a timeout occurs while waiting, or the status is changed
+to an unexpected value (e.g. `error``), the image creation fails.
+
+Possible values:
+    * A positive integer
+
+Related options:
+    * None
+
+""")),
     cfg.BoolOpt('cinder_api_insecure',
                 default=False,
-                help=_('Allow to perform insecure SSL requests to cinder')),
+                help=_("""
+Allow to perform insecure SSL requests to cinder.
+
+If this option is set to True, HTTPS endpoint connection is verified using the
+CA certificates file specified by ``cinder_ca_certificates_file`` option.
+
+Possible values:
+    * True
+    * False
+
+Related options:
+    * cinder_ca_certificates_file
+
+""")),
     cfg.StrOpt('cinder_store_auth_address',
                default=None,
-               help=_('The address where the Cinder authentication service '
-                      'is listening. If <None>, the cinder endpoint in the '
-                      'service catalog is used.')),
+               help=_("""
+The address where the cinder authentication service is listening.
+
+When all of ``cinder_store_auth_address``, ``cinder_store_user_name``,
+``cinder_store_project_name``, and ``cinder_store_password`` options are
+specified, the specified values are always used for the authentication.
+This is useful to hide the image volumes from users by storing them in a
+project/tenant specific to the image service. It also enables users to share
+the image volume among other projects under the control of glance's ACL.
+
+If either of these options are not set, the cinder endpoint is looked up
+from the service catalog, and current context's user and project are used.
+
+Possible values:
+    * A valid authentication service address, for example:
+      ``http://openstack.example.org/identity/v2.0``
+
+Related options:
+    * cinder_store_user_name
+    * cinder_store_password
+    * cinder_store_project_name
+
+""")),
     cfg.StrOpt('cinder_store_user_name',
                default=None,
-               help=_('User name to authenticate against Cinder. If <None>, '
-                      'the user of current context is used.')),
+               help=_("""
+User name to authenticate against cinder.
+
+This must be used with all the following related options. If any of these are
+not specified, the user of the current context is used.
+
+Possible values:
+    * A valid user name
+
+Related options:
+    * cinder_store_auth_address
+    * cinder_store_password
+    * cinder_store_project_name
+
+""")),
     cfg.StrOpt('cinder_store_password', secret=True,
-               default=None,
-               help=_('Password for the user authenticating against Cinder. '
-                      'If <None>, the current context auth token is used.')),
+               help=_("""
+Password for the user authenticating against cinder.
+
+This must be used with all the following related options. If any of these are
+not specified, the user of the current context is used.
+
+Possible values:
+    * A valid password for the user specified by ``cinder_store_user_name``
+
+Related options:
+    * cinder_store_auth_address
+    * cinder_store_user_name
+    * cinder_store_project_name
+
+""")),
     cfg.StrOpt('cinder_store_project_name',
                default=None,
-               help=_('Project name where the image is stored in Cinder. '
-                      'If <None>, the project in current context is used.')),
+               help=_("""
+Project name where the image volume is stored in cinder.
+
+If this configuration option is not set, the project in current context is
+used.
+
+This must be used with all the following related options. If any of these are
+not specified, the project of the current context is used.
+
+Possible values:
+    * A valid project name
+
+Related options:
+    * ``cinder_store_auth_address``
+    * ``cinder_store_user_name``
+    * ``cinder_store_password``
+
+""")),
     cfg.StrOpt('rootwrap_config',
                default='/etc/glance/rootwrap.conf',
-               help=_('Path to the rootwrap configuration file to use for '
-                      'running commands as root.')),
+               help=_("""
+Path to the rootwrap configuration file to use for running commands as root.
+
+The cinder store requires root privileges to operate the image volumes (for
+connecting to iSCSI/FC volumes and reading/writing the volume data, etc.).
+The configuration file should allow the required commands by cinder store and
+os-brick library.
+
+Possible values:
+    * Path to the rootwrap config file
+
+Related options:
+    * None
+
+""")),
+    cfg.StrOpt('cinder_volume_type',
+               default=None,
+               help=_("""
+Volume type that will be used for volume creation in cinder.
+
+Some cinder backends can have several volume types to optimize storage usage.
+Adding this option allows an operator to choose a specific volume type
+in cinder that can be optimized for images.
+
+If this is not set, then the default volume type specified in the cinder
+configuration will be used for volume creation.
+
+Possible values:
+    * A valid volume type from cinder
+
+Related options:
+    * None
+
+""")),
 ]
 
 
@@ -123,14 +335,14 @@ def get_cinderclient(conf, context=None):
             url = glance_store.cinder_endpoint_template % context.to_dict()
         else:
             info = glance_store.cinder_catalog_info
-            service_type, service_name, endpoint_type = info.split(':')
-            sc = {'serviceCatalog': context.service_catalog}
+            service_type, service_name, interface = info.split(':')
             try:
-                url = keystone_sc.ServiceCatalogV2(sc).url_for(
+                catalog = keystone_sc.ServiceCatalogV2(context.service_catalog)
+                url = catalog.url_for(
                     region_name=glance_store.cinder_os_region_name,
                     service_type=service_type,
                     service_name=service_name,
-                    endpoint_type=endpoint_type)
+                    interface=interface)
             except keystone_exc.EndpointNotFound:
                 reason = _("Failed to find Cinder from a service catalog.")
                 raise exceptions.BadStoreConfiguration(store_name="cinder",
@@ -167,10 +379,7 @@ class StoreLocation(glance_store.location.StoreLocation):
         return "cinder://%s" % self.volume_id
 
     def parse_uri(self, uri):
-        if not uri.startswith('cinder://'):
-            reason = _("URI must start with 'cinder://'")
-            LOG.info(reason)
-            raise exceptions.BadStoreUri(message=reason)
+        self.validate_schemas(uri, valid_schemas=('cinder://',))
 
         self.scheme = 'cinder'
         self.volume_id = uri[9:]
@@ -264,6 +473,7 @@ class Store(glance_store.driver.Store):
         attach_mode = 'rw' if mode == 'wb' else 'ro'
         device = None
         root_helper = get_root_helper()
+        priv_context.init(root_helper=shlex.split(root_helper))
         host = socket.gethostname()
         properties = connector.get_connector_properties(root_helper, host,
                                                         False, False)
@@ -284,10 +494,13 @@ class Store(glance_store.driver.Store):
             device = conn.connect_volume(connection_info['data'])
             volume.attach(None, None, attach_mode, host_name=host)
             volume = self._wait_volume_status(volume, 'attaching', 'in-use')
-            LOG.debug('Opening host device "%s"', device['path'])
-            with temporary_chown(device['path']), \
-                    open(device['path'], mode) as f:
-                yield f
+            if (connection_info['driver_volume_type'] == 'rbd' and
+               not conn.do_local_attach):
+                yield device['path']
+            else:
+                with temporary_chown(device['path']), \
+                        open(device['path'], mode) as f:
+                    yield f
         except Exception:
             LOG.exception(_LE('Exception while accessing to cinder volume '
                               '%(volume_id)s.'), {'volume_id': volume.id})
@@ -352,12 +565,12 @@ class Store(glance_store.driver.Store):
         where to find the image file, and returns a tuple of generator
         (for reading the image file) and image_size
 
-        :param location `glance_store.location.Location` object, supplied
+        :param location: `glance_store.location.Location` object, supplied
                         from glance_store.location.get_location_from_uri()
         :param offset: offset to start reading
         :param chunk_size: size to read, or None to get all the image
         :param context: Request context
-        :raises `glance_store.exceptions.NotFound` if image does not exist
+        :raises: `glance_store.exceptions.NotFound` if image does not exist
         """
 
         loc = location.store_location
@@ -373,11 +586,11 @@ class Store(glance_store.driver.Store):
             return (iterator, chunk_size or size)
         except cinder_exception.NotFound:
             reason = _("Failed to get image size due to "
-                       "volume can not be found: %s") % volume.id
+                       "volume can not be found: %s") % loc.volume_id
             LOG.error(reason)
             raise exceptions.NotFound(reason)
         except cinder_exception.ClientException as e:
-            msg = (_('Failed to get image volume %(volume_id): %(error)s')
+            msg = (_('Failed to get image volume %(volume_id)s: %(error)s')
                    % {'volume_id': loc.volume_id, 'error': e})
             LOG.error(msg)
             raise exceptions.BackendException(msg)
@@ -390,7 +603,7 @@ class Store(glance_store.driver.Store):
         :param location: `glance_store.location.Location` object, supplied
                         from glance_store.location.get_location_from_uri()
         :raises: `glance_store.exceptions.NotFound` if image does not exist
-        :rtype int
+        :rtype: int
         """
 
         loc = location.store_location
@@ -422,9 +635,9 @@ class Store(glance_store.driver.Store):
         :param context: The request context
         :param verifier: An object used to verify signatures for images
 
-        :retval tuple of URL in backing store, bytes written, checksum
+        :returns: tuple of URL in backing store, bytes written, checksum
                 and a dictionary with storage system specific information
-        :raises `glance_store.exceptions.Duplicate` if the image already
+        :raises: `glance_store.exceptions.Duplicate` if the image already
                 existed
         """
 
@@ -441,14 +654,17 @@ class Store(glance_store.driver.Store):
         metadata = {'glance_image_id': image_id,
                     'image_size': str(image_size),
                     'image_owner': owner}
-        LOG.debug('Creating a new volume: image_size=%d size_gb=%d',
-                  image_size, size_gb)
+        volume_type = self.conf.glance_store.cinder_volume_type
+        LOG.debug('Creating a new volume: image_size=%d size_gb=%d type=%s',
+                  image_size, size_gb, volume_type or 'None')
         if image_size == 0:
             LOG.info(_LI("Since image size is zero, we will be doing "
                          "resize-before-write for each GB which "
                          "will be considerably slower than normal."))
-        volume = client.volumes.create(size_gb, name=name, metadata=metadata)
+        volume = client.volumes.create(size_gb, name=name, metadata=metadata,
+                                       volume_type=volume_type)
         volume = self._wait_volume_status(volume, 'creating', 'available')
+        size_gb = volume.size
 
         failed = True
         need_extend = True
@@ -483,6 +699,7 @@ class Store(glance_store.driver.Store):
                         volume = self._wait_volume_status(volume,
                                                           'extending',
                                                           'available')
+                        size_gb = volume.size
                     except exceptions.BackendException:
                         raise exceptions.StorageFull()
 
@@ -525,11 +742,11 @@ class Store(glance_store.driver.Store):
         Takes a `glance_store.location.Location` object that indicates
         where to find the image file to delete
 
-        :location `glance_store.location.Location` object, supplied
+        :param location: `glance_store.location.Location` object, supplied
                   from glance_store.location.get_location_from_uri()
 
-        :raises NotFound if image does not exist
-        :raises Forbidden if cannot delete because of permissions
+        :raises: NotFound if image does not exist
+        :raises: Forbidden if cannot delete because of permissions
         """
         loc = location.store_location
         self._check_context(context)
