@@ -25,7 +25,6 @@ import math
 
 from oslo_config import cfg
 from oslo_utils import units
-import six
 from six.moves import urllib
 
 from glance_store import capabilities
@@ -52,25 +51,100 @@ LOG = logging.getLogger(__name__)
 
 _RBD_OPTS = [
     cfg.IntOpt('rbd_store_chunk_size', default=DEFAULT_CHUNKSIZE,
-               help=_('RADOS images will be chunked into objects of this size '
-                      '(in megabytes). For best performance, this should be '
-                      'a power of two.')),
+               min=1,
+               help=_("""
+Size, in megabytes, to chunk RADOS images into.
+
+Provide an integer value representing the size in megabytes to chunk
+Glance images into. The default chunk size is 8 megabytes. For optimal
+performance, the value should be a power of two.
+
+When Ceph's RBD object storage system is used as the storage backend
+for storing Glance images, the images are chunked into objects of the
+size set using this option. These chunked objects are then stored
+across the distributed block data store to use for Glance.
+
+Possible Values:
+    * Any positive integer value
+
+Related options:
+    * None
+
+""")),
     cfg.StrOpt('rbd_store_pool', default=DEFAULT_POOL,
-               help=_('RADOS pool in which images are stored.')),
+               help=_("""
+RADOS pool in which images are stored.
+
+When RBD is used as the storage backend for storing Glance images, the
+images are stored by means of logical grouping of the objects (chunks
+of images) into a ``pool``. Each pool is defined with the number of
+placement groups it can contain. The default pool that is used is
+'images'.
+
+More information on the RBD storage backend can be found here:
+http://ceph.com/planet/how-data-is-stored-in-ceph-cluster/
+
+Possible Values:
+    * A valid pool name
+
+Related options:
+    * None
+
+""")),
     cfg.StrOpt('rbd_store_user', default=DEFAULT_USER,
-               help=_('RADOS user to authenticate as (only applicable if '
-                      'using Cephx. If <None>, a default will be chosen based '
-                      'on the client. section in rbd_store_ceph_conf)')),
+               help=_("""
+RADOS user to authenticate as.
+
+This configuration option takes in the RADOS user to authenticate as.
+This is only needed when RADOS authentication is enabled and is
+applicable only if the user is using Cephx authentication. If the
+value for this option is not set by the user or is set to None, a
+default value will be chosen, which will be based on the client.
+section in rbd_store_ceph_conf.
+
+Possible Values:
+    * A valid RADOS user
+
+Related options:
+    * rbd_store_ceph_conf
+
+""")),
     cfg.StrOpt('rbd_store_ceph_conf', default=DEFAULT_CONFFILE,
-               help=_('Ceph configuration file path. '
-                      'If <None>, librados will locate the default config. '
-                      'If using cephx authentication, this file should '
-                      'include a reference to the right keyring '
-                      'in a client.<USER> section')),
+               help=_("""
+Ceph configuration file path.
+
+This configuration option takes in the path to the Ceph configuration
+file to be used. If the value for this option is not set by the user
+or is set to None, librados will locate the default configuration file
+which is located at /etc/ceph/ceph.conf. If using Cephx
+authentication, this file should include a reference to the right
+keyring in a client.<USER> section
+
+Possible Values:
+    * A valid path to a configuration file
+
+Related options:
+    * rbd_store_user
+
+""")),
     cfg.IntOpt('rados_connect_timeout', default=0,
-               help=_('Timeout value (in seconds) used when connecting to '
-                      'ceph cluster. If value <= 0, no timeout is set and '
-                      'default librados value is used.'))
+               help=_("""
+Timeout value for connecting to Ceph cluster.
+
+This configuration option takes in the timeout value in seconds used
+when connecting to the Ceph cluster i.e. it sets the time to wait for
+glance-api before closing the connection. This prevents glance-api
+hangups during the connection to RBD. If the value for this option
+is set to less than or equal to 0, no timeout is set and the default
+librados value is used.
+
+Possible Values:
+    * Any integer value
+
+Related options:
+    * None
+
+"""))
 ]
 
 
@@ -87,7 +161,7 @@ class StoreLocation(location.StoreLocation):
 
     def process_specs(self):
         # convert to ascii since librbd doesn't handle unicode
-        for key, value in six.iteritems(self.specs):
+        for key, value in self.specs.items():
             self.specs[key] = str(value)
         self.fsid = self.specs.get('fsid')
         self.pool = self.specs.get('pool')
@@ -108,12 +182,7 @@ class StoreLocation(location.StoreLocation):
 
     def parse_uri(self, uri):
         prefix = 'rbd://'
-        if not uri.startswith(prefix):
-            reason = _('URI must start with rbd://')
-            msg = _LI("Invalid URI: %s") % reason
-
-            LOG.info(msg)
-            raise exceptions.BadStoreUri(message=reason)
+        self.validate_schemas(uri, valid_schemas=(prefix,))
         # convert to ascii since librbd doesn't handle unicode
         try:
             ascii_uri = str(uri)
@@ -165,8 +234,7 @@ class ImageIterator(object):
                 with conn.open_ioctx(self.pool) as ioctx:
                     with rbd.Image(ioctx, self.name,
                                    snapshot=self.snapshot) as image:
-                        img_info = image.stat()
-                        size = img_info['size']
+                        size = image.size()
                         bytes_left = size
                         while bytes_left > 0:
                             length = min(self.chunk_size, bytes_left)
@@ -280,7 +348,7 @@ class Store(driver.Store):
 
         :param image_name: Image's name
 
-        :retval: `glance_store.rbd.StoreLocation` object
+        :returns: `glance_store.rbd.StoreLocation` object
         """
         librbd = rbd.RBD()
         features = conn.conf_get('rbd_default_features')
@@ -314,7 +382,7 @@ class Store(driver.Store):
                     if snapshot_name is not None:
                         with rbd.Image(ioctx, image_name) as image:
                             try:
-                                image.unprotect_snap(snapshot_name)
+                                self._unprotect_snapshot(image, snapshot_name)
                                 image.remove_snap(snapshot_name)
                             except rbd.ImageNotFound as exc:
                                 msg = (_("Snap Operating Exception "
@@ -348,6 +416,15 @@ class Store(driver.Store):
                     msg = _("RBD image %s does not exist") % image_name
                     raise exceptions.NotFound(message=msg)
 
+    def _unprotect_snapshot(self, image, snap_name):
+        try:
+            image.unprotect_snap(snap_name)
+        except rbd.InvalidArgument:
+            # NOTE(slaweq): if snapshot was unprotected already, rbd library
+            # raises InvalidArgument exception without any "clear" message.
+            # Such exception is not dangerous for us so it will be just logged
+            LOG.debug("Snapshot %s is unprotected already" % snap_name)
+
     @capabilities.check
     def add(self, image_id, image_file, image_size, context=None,
             verifier=None):
@@ -361,7 +438,7 @@ class Store(driver.Store):
         :param image_size: The size of the image data to write, in bytes
         :param verifier: An object used to verify signatures for images
 
-        :retval: tuple of URL in backing store, bytes written, checksum
+        :returns: tuple of URL in backing store, bytes written, checksum
                 and a dictionary with storage system specific information
         :raises: `glance_store.exceptions.Duplicate` if the image already
                 existed
