@@ -19,8 +19,10 @@ import hashlib
 import logging
 import math
 
-from keystoneclient import exceptions as keystone_exc
-from keystoneclient import service_catalog as keystone_sc
+from keystoneauth1.access import service_catalog as keystone_sc
+from keystoneauth1 import identity as ks_identity
+from keystoneauth1 import session as ks_session
+from keystoneclient.v3 import client as ks_client
 from oslo_config import cfg
 from oslo_utils import encodeutils
 from oslo_utils import excutils
@@ -33,11 +35,8 @@ try:
 except ImportError:
     swiftclient = None
 
-from keystoneclient.auth.identity import v3 as ks_v3
-from keystoneclient import session as ks_session
-from keystoneclient.v3 import client as ks_client
-
 import glance_store
+from glance_store._drivers.swift import buffered
 from glance_store._drivers.swift import connection_manager
 from glance_store._drivers.swift import utils as sutils
 from glance_store import capabilities
@@ -56,28 +55,133 @@ ONE_MB = units.k * units.Ki  # Here we used the mixed meaning of MB
 
 _SWIFT_OPTS = [
     cfg.BoolOpt('swift_store_auth_insecure', default=False,
-                help=_('If True, swiftclient won\'t check for a valid SSL '
-                       'certificate when authenticating.')),
+                help=_("""
+Set verification of the server certificate.
+
+This boolean determines whether or not to verify the server
+certificate. If this option is set to True, swiftclient won't check
+for a valid SSL certificate when authenticating. If the option is set
+to False, then the default CA truststore is used for verification.
+
+Possible values:
+    * True
+    * False
+
+Related options:
+    * swift_store_cacert
+
+""")),
     cfg.StrOpt('swift_store_cacert',
-               help=_('A string giving the CA certificate file to use in '
-                      'SSL connections for verifying certs.')),
+               sample_default='/etc/ssl/certs/ca-certificates.crt',
+               help=_("""
+Path to the CA bundle file.
+
+This configuration option enables the operator to specify the path to
+a custom Certificate Authority file for SSL verification when
+connecting to Swift.
+
+Possible values:
+    * A valid path to a CA file
+
+Related options:
+    * swift_store_auth_insecure
+
+""")),
     cfg.StrOpt('swift_store_region',
-               help=_('The region of the swift endpoint to be used for '
-                      'single tenant. This setting is only necessary if the '
-                      'tenant has multiple swift endpoints.')),
+               sample_default='RegionTwo',
+               help=_("""
+The region of Swift endpoint to use by Glance.
+
+Provide a string value representing a Swift region where Glance
+can connect to for image storage. By default, there is no region
+set.
+
+When Glance uses Swift as the storage backend to store images
+for a specific tenant that has multiple endpoints, setting of a
+Swift region with ``swift_store_region`` allows Glance to connect
+to Swift in the specified region as opposed to a single region
+connectivity.
+
+This option can be configured for both single-tenant and
+multi-tenant storage.
+
+NOTE: Setting the region with ``swift_store_region`` is
+tenant-specific and is necessary ``only if`` the tenant has
+multiple endpoints across different regions.
+
+Possible values:
+    * A string value representing a valid Swift region.
+
+Related Options:
+    * None
+
+""")),
     cfg.StrOpt('swift_store_endpoint',
-               help=_('If set, the configured endpoint will be used. If '
-                      'None, the storage url from the auth response will be '
-                      'used.')),
+               sample_default="""\
+https://swift.openstack.example.org/v1/path_not_including_container\
+_name\
+""",
+               help=_("""
+The URL endpoint to use for Swift backend storage.
+
+Provide a string value representing the URL endpoint to use for
+storing Glance images in Swift store. By default, an endpoint
+is not set and the storage URL returned by ``auth`` is used.
+Setting an endpoint with ``swift_store_endpoint`` overrides the
+storage URL and is used for Glance image storage.
+
+NOTE: The URL should include the path up to, but excluding the
+container. The location of an object is obtained by appending
+the container and object to the configured URL.
+
+Possible values:
+    * String value representing a valid URL path up to a Swift container
+
+Related Options:
+    * None
+
+""")),
     cfg.StrOpt('swift_store_endpoint_type', default='publicURL',
-               help=_('A string giving the endpoint type of the swift '
-                      'service to use (publicURL, adminURL or internalURL). '
-                      'This setting is only used if swift_store_auth_version '
-                      'is 2.')),
-    cfg.StrOpt('swift_store_service_type', default='object-store',
-               help=_('A string giving the service type of the swift service '
-                      'to use. This setting is only used if '
-                      'swift_store_auth_version is 2.')),
+               choices=('publicURL', 'adminURL', 'internalURL'),
+               help=_("""
+Endpoint Type of Swift service.
+
+This string value indicates the endpoint type to use to fetch the
+Swift endpoint. The endpoint type determines the actions the user will
+be allowed to perform, for instance, reading and writing to the Store.
+This setting is only used if swift_store_auth_version is greater than
+1.
+
+Possible values:
+    * publicURL
+    * adminURL
+    * internalURL
+
+Related options:
+    * swift_store_endpoint
+
+""")),
+    cfg.StrOpt('swift_store_service_type',
+               default='object-store',
+               help=_("""
+Type of Swift service to use.
+
+Provide a string value representing the service type to use for
+storing images while using Swift backend storage. The default
+service type is set to ``object-store``.
+
+NOTE: If ``swift_store_auth_version`` is set to 2, the value for
+this configuration option needs to be ``object-store``. If using
+a higher version of Keystone or a different auth scheme, this
+option may be modified.
+
+Possible values:
+    * A string representing a valid service type for Swift storage.
+
+Related Options:
+    * None
+
+""")),
     cfg.StrOpt('swift_store_container',
                default=DEFAULT_CONTAINER,
                help=_("""
@@ -101,9 +205,6 @@ the container ``glance_fda``. All dashes in the UUID are included when
 creating the container name but do not count toward the character limit, so
 when N=10 the container name would be ``glance_fdae39a1-ba.``
 
-Services which consume this:
-    * None (directly consumed by the Swift driver)
-
 Possible values:
     * If using single container, this configuration option can be any string
       that is a valid swift container name in Glance's Swift account
@@ -127,7 +228,7 @@ Swift has an upper limit on the size of a single uploaded object. By default,
 this is 5GB. To upload objects bigger than this limit, objects are segmented
 into multiple smaller objects that are tied together with a manifest file.
 For more detail, refer to
-http://docs.openstack.org/developer/swift/overview_large_objects.html
+https://docs.openstack.org/swift/latest/overview_large_objects.html
 
 This configuration option specifies the size threshold over which the Swift
 driver will start segmenting image data into multiple smaller files.
@@ -136,9 +237,6 @@ Currently, the Swift driver only supports creating Dynamic Large Objects.
 NOTE: This should be set by taking into account the large object limit
 enforced by the Swift cluster in consideration.
 
-Services which consume this:
-    * None (directly consumed by the Swift driver)
-
 Possible values:
     * A positive integer that is less than or equal to the large object limit
       enforced by the Swift cluster in consideration.
@@ -162,9 +260,6 @@ For example: if ``swift_store_large_object_size`` is 5GB and
 segmented into 7 segments where the first six segments will be 1GB in size and
 the seventh segment will be 0.2GB.
 
-Services which consume this:
-    * None (directly consumed by the Swift driver)
-
 Possible values:
     * A positive integer that is less than or equal to the large object limit
       enforced by Swift cluster in consideration.
@@ -182,9 +277,6 @@ exist, it will be created provided this configuration option is set to True.
 By default, it won't be created. This behavior is applicable for both single
 and multiple containers mode.
 
-Services which consume this:
-    * None (directly consumed by the Swift driver)
-
 Possible values:
     * True
     * False
@@ -202,15 +294,16 @@ in tenant specific Swift accounts. If this is disabled, Glance stores all
 images in its own account. More details multi-tenant store can be found at
 https://wiki.openstack.org/wiki/GlanceSwiftTenantSpecificStorage
 
-Services which consume this:
-    * None (directly consumed by the Swift driver)
+NOTE: If using multi-tenant swift store, please make sure
+that you do not set a swift configuration file with the
+'swift_store_config_file' option.
 
 Possible values:
     * True
     * False
 
 Related options:
-    * None
+    * swift_store_config_file
 
 """)),
     cfg.IntOpt('swift_store_multiple_containers_seed',
@@ -232,9 +325,6 @@ https://specs.openstack.org/openstack/glance-specs/specs/kilo/swift-store-multip
 
 NOTE: This is used only when swift_store_multi_tenant is disabled.
 
-Services which consume this:
-    * None (directly consumed only by the Swift driver)
-
 Possible values:
     * A non-negative integer less than or equal to 32
 
@@ -245,32 +335,145 @@ Related options:
 
 """)),
     cfg.ListOpt('swift_store_admin_tenants', default=[],
-                help=_('A list of tenants that will be granted read/write '
-                       'access on all Swift containers created by Glance in '
-                       'multi-tenant mode.')),
-    cfg.BoolOpt('swift_store_ssl_compression', default=True,
-                help=_('If set to False, disables SSL layer compression of '
-                       'https swift requests. Setting to False may improve '
-                       'performance for images which are already in a '
-                       'compressed format, eg qcow2.')),
-    cfg.IntOpt('swift_store_retry_get_count', default=0,
-               help=_('The number of times a Swift download will be retried '
-                      'before the request fails.')),
-    cfg.IntOpt('swift_store_expire_soon_interval', default=60,
-               help=_('The period of time (in seconds) before token '
-                      'expiration when glance_store will try to request new '
-                      'user token. Default value 60 sec means that if token '
-                      'is going to expire in 1 min then glance_store requests '
-                      'new user token.')),
-    cfg.BoolOpt('swift_store_use_trusts', default=True,
-                help=_('If set to True create a trust for each add/get '
-                       'request to Multi-tenant store in order to prevent '
-                       'authentication token to be expired during '
-                       'uploading/downloading data. If set to False then user '
-                       'token is used for Swift connection (so no overhead on '
-                       'trust creation). Please note that this '
-                       'option is considered only and only if '
-                       'swift_store_multi_tenant=True'))
+                help=_("""
+List of tenants that will be granted admin access.
+
+This is a list of tenants that will be granted read/write access on
+all Swift containers created by Glance in multi-tenant mode. The
+default value is an empty list.
+
+Possible values:
+    * A comma separated list of strings representing UUIDs of Keystone
+      projects/tenants
+
+Related options:
+    * None
+
+""")),
+    cfg.BoolOpt('swift_store_ssl_compression',
+                default=True,
+                help=_("""
+SSL layer compression for HTTPS Swift requests.
+
+Provide a boolean value to determine whether or not to compress
+HTTPS Swift requests for images at the SSL layer. By default,
+compression is enabled.
+
+When using Swift as the backend store for Glance image storage,
+SSL layer compression of HTTPS Swift requests can be set using
+this option. If set to False, SSL layer compression of HTTPS
+Swift requests is disabled. Disabling this option may improve
+performance for images which are already in a compressed format,
+for example, qcow2.
+
+Possible values:
+    * True
+    * False
+
+Related Options:
+    * None
+
+""")),
+    cfg.IntOpt('swift_store_retry_get_count',
+               default=0,
+               min=0,
+               help=_("""
+The number of times a Swift download will be retried before the
+request fails.
+
+Provide an integer value representing the number of times an image
+download must be retried before erroring out. The default value is
+zero (no retry on a failed image download). When set to a positive
+integer value, ``swift_store_retry_get_count`` ensures that the
+download is attempted this many more times upon a download failure
+before sending an error message.
+
+Possible values:
+    * Zero
+    * Positive integer value
+
+Related Options:
+    * None
+
+""")),
+    cfg.IntOpt('swift_store_expire_soon_interval',
+               min=0,
+               default=60,
+               help=_("""
+Time in seconds defining the size of the window in which a new
+token may be requested before the current token is due to expire.
+
+Typically, the Swift storage driver fetches a new token upon the
+expiration of the current token to ensure continued access to
+Swift. However, some Swift transactions (like uploading image
+segments) may not recover well if the token expires on the fly.
+
+Hence, by fetching a new token before the current token expiration,
+we make sure that the token does not expire or is close to expiry
+before a transaction is attempted. By default, the Swift storage
+driver requests for a new token 60 seconds or less before the
+current token expiration.
+
+Possible values:
+    * Zero
+    * Positive integer value
+
+Related Options:
+    * None
+
+""")),
+    cfg.BoolOpt('swift_store_use_trusts',
+                default=True,
+                help=_("""
+Use trusts for multi-tenant Swift store.
+
+This option instructs the Swift store to create a trust for each
+add/get request when the multi-tenant store is in use. Using trusts
+allows the Swift store to avoid problems that can be caused by an
+authentication token expiring during the upload or download of data.
+
+By default, ``swift_store_use_trusts`` is set to ``True``(use of
+trusts is enabled). If set to ``False``, a user token is used for
+the Swift connection instead, eliminating the overhead of trust
+creation.
+
+NOTE: This option is considered only when
+``swift_store_multi_tenant`` is set to ``True``
+
+Possible values:
+    * True
+    * False
+
+Related options:
+    * swift_store_multi_tenant
+
+""")),
+    cfg.BoolOpt('swift_buffer_on_upload',
+                default=False,
+                help=_("""
+Buffer image segments before upload to Swift.
+
+Provide a boolean value to indicate whether or not Glance should
+buffer image data to disk while uploading to swift. This enables
+Glance to resume uploads on error.
+
+NOTES:
+When enabling this option, one should take great care as this
+increases disk usage on the API node. Be aware that depending
+upon how the file system is configured, the disk space used
+for buffering may decrease the actual disk space available for
+the glance image cache.  Disk utilization will cap according to
+the following equation:
+(``swift_store_large_object_chunk_size`` * ``workers`` * 1000)
+
+Possible values:
+    * True
+    * False
+
+Related options:
+    * swift_upload_buffer_dir
+
+"""))
 ]
 
 
@@ -482,8 +685,8 @@ class StoreLocation(location.StoreLocation):
             raise exceptions.BadStoreUri(message=reason)
 
         pieces = urllib.parse.urlparse(uri)
-        assert pieces.scheme in ('swift', 'swift+http', 'swift+https',
-                                 'swift+config')
+        self.validate_schemas(uri, valid_schemas=(
+            'swift://', 'swift+http://', 'swift+https://', 'swift+config://'))
 
         self.scheme = pieces.scheme
         netloc = pieces.netloc
@@ -524,9 +727,22 @@ class StoreLocation(location.StoreLocation):
 
 
 def Store(conf):
+    # NOTE(dharinic): Multi-tenant store cannot work with swift config
+    if conf.glance_store.swift_store_multi_tenant:
+        if (conf.glance_store.default_store == 'swift+config' or
+           sutils.is_multiple_swift_store_accounts_enabled(conf)):
+            msg = _("Swift multi-tenant store cannot be configured to "
+                    "work with swift+config. The options "
+                    "'swift_store_multi_tenant' and "
+                    "'swift_store_config_file' are mutually exclusive. "
+                    "If you inted to use multi-tenant swift store, please "
+                    "make sure that you have not set a swift configuration "
+                    "file with the 'swift_store_config_file' option.")
+            raise exceptions.BadStoreConfiguration(store_name="swift",
+                                                   reason=msg)
     try:
-        conf.register_opts(_SWIFT_OPTS + sutils.swift_opts,
-                           group='glance_store')
+        conf.register_opts(_SWIFT_OPTS + sutils.swift_opts +
+                           buffered.BUFFERING_OPTS, group='glance_store')
     except cfg.DuplicateOptError:
         pass
 
@@ -534,7 +750,7 @@ def Store(conf):
         return MultiTenantStore(conf)
     return SingleTenantStore(conf)
 
-Store.OPTIONS = _SWIFT_OPTS + sutils.swift_opts
+Store.OPTIONS = _SWIFT_OPTS + sutils.swift_opts + buffered.BUFFERING_OPTS
 
 
 def _is_slo(slo_header):
@@ -572,6 +788,14 @@ class BaseStore(driver.Store):
             msg = _("Missing dependency python_swiftclient.")
             raise exceptions.BadStoreConfiguration(store_name="swift",
                                                    reason=msg)
+
+        if glance_conf.swift_buffer_on_upload:
+            buffer_dir = glance_conf.swift_upload_buffer_dir
+            if buffered.validate_buffering(buffer_dir):
+                self.reader_class = buffered.BufferedReader
+        else:
+            self.reader_class = ChunkReader
+
         super(BaseStore, self).configure(re_raise_bsc=re_raise_bsc)
 
     def _get_object(self, location, manager, start=None):
@@ -601,8 +825,8 @@ class BaseStore(driver.Store):
         # initialize manager to receive valid connections
         allow_retry = \
             self.conf.glance_store.swift_store_retry_get_count > 0
-        with get_manager_for_store(self, location, context,
-                                   allow_reauth=allow_retry) as manager:
+        with self.get_manager(location, context,
+                              allow_reauth=allow_retry) as manager:
             (resp_headers, resp_body) = self._get_object(location,
                                                          manager=manager)
 
@@ -658,8 +882,8 @@ class BaseStore(driver.Store):
         # initialize a manager with re-auth if image need to be splitted
         need_chunks = (image_size == 0) or (
             image_size >= self.large_object_size)
-        with get_manager_for_store(self, location, context,
-                                   allow_reauth=need_chunks) as manager:
+        with self.get_manager(location, context,
+                              allow_reauth=need_chunks) as manager:
 
             self._create_container_if_missing(location.container,
                                               manager.get_connection())
@@ -692,8 +916,11 @@ class BaseStore(driver.Store):
                         # image_size == 0 is when we don't know the size
                         # of the image. This can occur with older clients
                         # that don't inspect the payload size.
-                        LOG.debug("Cannot determine image size. Adding as a "
-                                  "segmented object to Swift.")
+                        LOG.debug("Cannot determine image size because it is "
+                                  "either not provided in the request or "
+                                  "chunked-transfer encoding is used. "
+                                  "Adding image as a segmented object to "
+                                  "Swift.")
                         total_chunks = '?'
 
                     checksum = hashlib.md5()
@@ -712,42 +939,45 @@ class BaseStore(driver.Store):
                             content_length = chunk_size
 
                         chunk_name = "%s-%05d" % (location.obj, chunk_id)
-                        reader = ChunkReader(image_file, checksum, chunk_size,
-                                             verifier)
-                        if reader.is_zero_size is True:
-                            LOG.debug('Not writing zero-length chunk.')
-                            break
-                        try:
-                            chunk_etag = manager.get_connection().put_object(
-                                location.container, chunk_name, reader,
-                                content_length=content_length)
-                            written_chunks.append(chunk_name)
-                        except Exception:
-                            # Delete orphaned segments from swift backend
-                            with excutils.save_and_reraise_exception():
-                                reason = _LE("Error during chunked upload to "
-                                             "backend, deleting stale chunks")
-                                LOG.exception(reason)
-                                self._delete_stale_chunks(
-                                    manager.get_connection(),
-                                    location.container,
-                                    written_chunks)
 
-                        bytes_read = reader.bytes_read
-                        msg = ("Wrote chunk %(chunk_name)s (%(chunk_id)d/"
-                               "%(total_chunks)s) of length %(bytes_read)d "
-                               "to Swift returning MD5 of content: "
-                               "%(chunk_etag)s" %
-                               {'chunk_name': chunk_name,
-                                'chunk_id': chunk_id,
-                                'total_chunks': total_chunks,
-                                'bytes_read': bytes_read,
-                                'chunk_etag': chunk_etag})
-                        LOG.debug(msg)
+                        with self.reader_class(image_file, checksum,
+                                               chunk_size, verifier) as reader:
+                            if reader.is_zero_size is True:
+                                LOG.debug('Not writing zero-length chunk.')
+                                break
+
+                            try:
+                                chunk_etag = \
+                                    manager.get_connection().put_object(
+                                        location.container,
+                                        chunk_name, reader,
+                                        content_length=content_length)
+                                written_chunks.append(chunk_name)
+                            except Exception:
+                                # Delete orphaned segments from swift backend
+                                with excutils.save_and_reraise_exception():
+                                    LOG.error(_("Error during chunked upload "
+                                                "to backend, deleting stale "
+                                                "chunks."))
+                                    self._delete_stale_chunks(
+                                        manager.get_connection(),
+                                        location.container,
+                                        written_chunks)
+
+                            bytes_read = reader.bytes_read
+                            msg = ("Wrote chunk %(chunk_name)s (%(chunk_id)d/"
+                                   "%(total_chunks)s) of length %(bytes_read)"
+                                   "d to Swift returning MD5 of content: "
+                                   "%(chunk_etag)s" %
+                                   {'chunk_name': chunk_name,
+                                    'chunk_id': chunk_id,
+                                    'total_chunks': total_chunks,
+                                    'bytes_read': bytes_read,
+                                    'chunk_etag': chunk_etag})
+                            LOG.debug(msg)
 
                         chunk_id += 1
                         combined_chunks_size += bytes_read
-
                     # In the case we have been given an unknown image size,
                     # set the size to the total size of the combined chunks.
                     if image_size == 0:
@@ -906,7 +1136,7 @@ class BaseStore(driver.Store):
         :param auth_token: auth token
         :param storage_url: swift storage url
         :return: swiftclient connection that allows to request container and
-        others
+                 others
         """
         # initialize a connection
         return swiftclient.Connection(
@@ -916,6 +1146,23 @@ class BaseStore(driver.Store):
             ssl_compression=self.ssl_compression,
             cacert=self.cacert)
 
+    def get_manager(self, store_location, context=None, allow_reauth=False):
+        """Return appropriate connection manager for store
+
+        The method detects store type (singletenant or multitenant) and returns
+        appropriate connection manager (singletenant or multitenant) that
+        allows to request swiftclient connections.
+
+        :param store_location: StoreLocation object that define image location
+        :param context: user context
+        :param allow_reauth: defines if we allow re-authentication when user
+            token is expired and refresh swift connection
+
+        :return: connection manager for store
+        """
+        msg = _("There is no Connection Manager implemented for %s class.")
+        raise NotImplementedError(msg % self.__class__.__name__)
+
 
 class SingleTenantStore(BaseStore):
     EXAMPLE_URL = "swift://<USER>:<KEY>@<AUTH_ADDRESS>/<CONTAINER>/<FILE>"
@@ -1074,18 +1321,25 @@ class SingleTenantStore(BaseStore):
             raise exceptions.BadStoreUri(message=reason)
 
         # initialize a keystone plugin for swift admin with creds
-        password = ks_v3.Password(auth_url=auth_url,
-                                  username=user,
-                                  password=location.key,
-                                  project_name=tenant_name,
-                                  user_domain_id=self.user_domain_id,
-                                  user_domain_name=self.user_domain_name,
-                                  project_domain_id=self.project_domain_id,
-                                  project_domain_name=self.project_domain_name)
-        sess = ks_session.Session(auth=password)
+        password = ks_identity.V3Password(
+            auth_url=auth_url,
+            username=user,
+            password=location.key,
+            project_name=tenant_name,
+            user_domain_id=self.user_domain_id,
+            user_domain_name=self.user_domain_name,
+            project_domain_id=self.project_domain_id,
+            project_domain_name=self.project_domain_name)
 
+        sess = ks_session.Session(auth=password)
         return ks_client.Client(session=sess)
 
+    def get_manager(self, store_location, context=None, allow_reauth=False):
+        return connection_manager.SingleTenantConnectionManager(self,
+                                                                store_location,
+                                                                context,
+                                                                allow_reauth)
+
 
 class MultiTenantStore(BaseStore):
     EXAMPLE_URL = "swift://<SWIFT_URL>/<CONTAINER>/<FILE>"
@@ -1103,10 +1357,10 @@ class MultiTenantStore(BaseStore):
                                                    reason=reason)
         self.storage_url = self.conf_endpoint
         if not self.storage_url:
-            sc = {'serviceCatalog': context.service_catalog}
-            self.storage_url = keystone_sc.ServiceCatalogV2(sc).url_for(
-                service_type=self.service_type, region_name=self.region,
-                endpoint_type=self.endpoint_type)
+            catalog = keystone_sc.ServiceCatalogV2(context.service_catalog)
+            self.storage_url = catalog.url_for(service_type=self.service_type,
+                                               region_name=self.region,
+                                               interface=self.endpoint_type)
 
         if self.storage_url.startswith('http://'):
             self.scheme = 'swift+http'
@@ -1167,15 +1421,8 @@ class MultiTenantStore(BaseStore):
         return StoreLocation(specs, self.conf)
 
     def get_connection(self, location, context=None):
-        try:
-            storage_url = self._get_endpoint(context)
-        except (exceptions.BadStoreConfiguration,
-                keystone_exc.EndpointNotFound) as e:
-            LOG.debug("Cannot obtain swift endpoint url from Service Catalog: "
-                      "%s. Use url stored in database.", e)
-            storage_url = location.swift_url
         return swiftclient.Connection(
-            preauthurl=storage_url,
+            preauthurl=location.swift_url,
             preauthtoken=context.auth_token,
             insecure=self.insecure,
             ssl_compression=self.ssl_compression,
@@ -1202,9 +1449,9 @@ class MultiTenantStore(BaseStore):
             'project_domain_name')
 
         # create client for multitenant user(trustor)
-        trustor_auth = ks_v3.Token(auth_url=auth_address,
-                                   token=context.auth_token,
-                                   project_id=context.tenant)
+        trustor_auth = ks_identity.V3Token(auth_url=auth_address,
+                                           token=context.auth_token,
+                                           project_id=context.tenant)
         trustor_sess = ks_session.Session(auth=trustor_auth)
         trustor_client = ks_client.Client(session=trustor_sess)
         auth_ref = trustor_client.session.auth.get_auth_ref(trustor_sess)
@@ -1212,14 +1459,15 @@ class MultiTenantStore(BaseStore):
 
         # create client for trustee - glance user specified in swift config
         tenant_name, user = user.split(':')
-        password = ks_v3.Password(auth_url=auth_address,
-                                  username=user,
-                                  password=key,
-                                  project_name=tenant_name,
-                                  user_domain_id=user_domain_id,
-                                  user_domain_name=user_domain_name,
-                                  project_domain_id=project_domain_id,
-                                  project_domain_name=project_domain_name)
+        password = ks_identity.V3Password(
+            auth_url=auth_address,
+            username=user,
+            password=key,
+            project_name=tenant_name,
+            user_domain_id=user_domain_id,
+            user_domain_name=user_domain_name,
+            project_domain_id=project_domain_id,
+            project_domain_name=project_domain_name)
         trustee_sess = ks_session.Session(auth=password)
         trustee_client = ks_client.Client(session=trustee_sess)
 
@@ -1234,7 +1482,7 @@ class MultiTenantStore(BaseStore):
         ).id
         # initialize a new client with trust and trustee credentials
         # create client for glance trustee user
-        client_password = ks_v3.Password(
+        client_password = ks_identity.V3Password(
             auth_url=auth_address,
             username=user,
             password=key,
@@ -1249,6 +1497,17 @@ class MultiTenantStore(BaseStore):
         client_sess = ks_session.Session(auth=client_password)
         return ks_client.Client(session=client_sess)
 
+    def get_manager(self, store_location, context=None, allow_reauth=False):
+        # if global toggle is turned off then do not allow re-authentication
+        # with trusts
+        if not self.conf.glance_store.swift_store_use_trusts:
+            allow_reauth = False
+
+        return connection_manager.MultiTenantConnectionManager(self,
+                                                               store_location,
+                                                               context,
+                                                               allow_reauth)
+
 
 class ChunkReader(object):
     def __init__(self, fd, checksum, total, verifier=None):
@@ -1280,33 +1539,8 @@ class ChunkReader(object):
             self.verifier.update(result)
         return result
 
+    def __enter__(self):
+        return self
 
-def get_manager_for_store(store, store_location,
-                          context=None,
-                          allow_reauth=False):
-    """Return appropriate connection manager for store
-
-    The method detects store type (singletenant or multitenant) and returns
-    appropriate connection manager (singletenant or multitenant) that allows
-    to request swiftclient connections.
-    :param store: store that needs swift connections
-    :param store_location: StoreLocation object that define image location
-    :param context: user context
-    :param allow_reauth: defines if we allow re-authentication when user token
-    is expired and refresh swift connection
-    :return: connection manager for store
-    """
-    if store.__class__ == SingleTenantStore:
-        return connection_manager.SingleTenantConnectionManager(
-            store, store_location, context, allow_reauth)
-    elif store.__class__ == MultiTenantStore:
-        # if global toggle is turned off then do not allow re-authentication
-        # with trusts
-        if not store.conf.glance_store.swift_store_use_trusts:
-            allow_reauth = False
-        return connection_manager.MultiTenantConnectionManager(
-            store, store_location, context, allow_reauth)
-    else:
-        raise NotImplementedError(_("There is no Connection Manager "
-                                    "implemented for %s class.") %
-                                  store.__class__.__name__)
+    def __exit__(self, type, value, traceback):
+        pass
