@@ -59,9 +59,9 @@ DS_URL_PREFIX = '/folder'
 STORE_SCHEME = 'vsphere'
 
 _VMWARE_OPTS = [
-    cfg.StrOpt('vmware_server_host',
-               sample_default='127.0.0.1',
-               help=_("""
+    cfg.HostAddressOpt('vmware_server_host',
+                       sample_default='127.0.0.1',
+                       help=_("""
 Address of the ESX/ESXi or vCenter Server target system.
 
 This configuration option sets the address of the ESX/ESXi or vCenter
@@ -320,11 +320,7 @@ class StoreLocation(location.StoreLocation):
     #    return path.startswith(os.path.join(DS_URL_PREFIX, sdir))
 
     def parse_uri(self, uri):
-        if not uri.startswith('%s://' % STORE_SCHEME):
-            reason = (_("URI %(uri)s must start with %(scheme)s://") %
-                      {'uri': uri, 'scheme': STORE_SCHEME})
-            LOG.info(reason)
-            raise exceptions.BadStoreUri(message=reason)
+        self.validate_schemas(uri, valid_schemas=('%s://' % STORE_SCHEME,))
         (self.scheme, self.server_host,
          path, params, query, fragment) = urllib.parse.urlparse(uri)
         if not query:
@@ -500,7 +496,7 @@ class Store(glance_store.Store):
 
     def select_datastore(self, image_size):
         """Select a datastore with free space larger than image size."""
-        for k, v in sorted(six.iteritems(self.datastores), reverse=True):
+        for k, v in sorted(self.datastores.items(), reverse=True):
             max_ds = None
             max_fs = 0
             for ds in v:
@@ -545,7 +541,7 @@ class Store(glance_store.Store):
         :param image_file: The image data to write, as a file-like object
         :param image_size: The size of the image data to write, in bytes
         :param verifier: An object used to verify signatures for images
-        :retval tuple of URL in backing store, bytes written, checksum
+        :returns: tuple of URL in backing store, bytes written, checksum
                 and a dictionary with storage system specific information
         :raises: `glance.common.exceptions.Duplicate` if the image already
                 existed
