@@ -38,11 +38,12 @@ credentials and is **not** user-facing.
 """
 
 import logging
-import urlparse
 
-from oslo.config import cfg
+from oslo_config import cfg
+from six.moves import urllib
 
 from glance_store import exceptions
+from glance_store.i18n import _
 
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
@@ -65,12 +66,10 @@ def get_location_from_uri(uri, conf=CONF):
         swift://example.com/container/obj-id
         swift://user:account:pass@authurl.com/container/obj-id
         swift+http://user:account:pass@authurl.com/container/obj-id
-        s3://accesskey:secretkey@s3.amazonaws.com/bucket/key-id
-        s3+https://accesskey:secretkey@s3.amazonaws.com/bucket/key-id
         file:///var/lib/glance/images/1
         cinder://volume-id
     """
-    pieces = urlparse.urlparse(uri)
+    pieces = urllib.parse.urlparse(uri)
     if pieces.scheme not in SCHEME_TO_CLS_MAP.keys():
         raise exceptions.UnknownScheme(scheme=pieces.scheme)
     scheme_info = SCHEME_TO_CLS_MAP[pieces.scheme]
@@ -169,3 +168,17 @@ class StoreLocation(object):
         """
         raise NotImplementedError("StoreLocation subclass must implement "
                                   "parse_uri()")
+
+    @staticmethod
+    def validate_schemas(uri, valid_schemas):
+        """check if uri scheme is one of valid_schemas
+        generate exception otherwise
+        """
+        for valid_schema in valid_schemas:
+            if uri.startswith(valid_schema):
+                return
+
+        reason = _("Location URI must start with one of the following "
+                   "schemas: %s") % ', '.join(valid_schemas)
+        LOG.warning(reason)
+        raise exceptions.BadStoreUri(message=reason)
